<?php

///////////////////////////////////////////////////////////////////////////

require_once 'lib/hashed_array.php';
require_once 'lib/tv/abstract_tv.php';
require_once 'lib/tv/default_epg_item.php';
require_once 'lib/tv/epg_iterator.php';

require_once 'sunduk_channel.php';

///////////////////////////////////////////////////////////////////////////

class SundukTv extends AbstractTv {

    private $session;
    
    ///////////////////////////////////////////////////////////////////////

    public function __construct($session) {
        $this->session = $session;

        parent::__construct(
                AbstractTv::MODE_CHANNELS_1_TO_N, true, false);
    }

    public function get_fav_icon_url() {
        return $this->session->get_icon('favorites.png');
    }

    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////

    protected function load_channels(&$plugin_cookies) {
        $this->session->check_logged_in();

        $settings = $this->session->get_settings();
        $buffering_ms = $settings->http_caching->value;

        $this->channels = new HashedArray();
        $this->groups = new HashedArray();

        $this->groups->put(
                new FavoritesGroup(
                $this, '__favorites', 'Избранное', $this->session->get_icon('favorites.png')));

        $this->groups->put(
                new AllChannelsGroup(
                $this, 'Все каналы', $this->session->get_icon('all.png')));

        $num_groups = 0;
        $num_channels = 0;
        $num_protected = 0;
        $num_have_archive = 0;

        $channel_list = $this->session->get_channel_list();
        foreach ($channel_list->groups as $g) {
            if (count($g->channels) == 0) {
                continue;
            }
            $sunduk_group = new DefaultGroup(
                    $g->id, $g->name, $this->session->get_group_icon($g->id));
            $this->groups->put($sunduk_group);
            $num_groups++;

            foreach ($g->channels as $c) {
                $have_archive = isset($c->have_archive) ?
                        ($c->have_archive == 1) : false;
                $is_protected = isset($c->protected) ?
                        ($c->protected == 1) : false;

                $timeshift_hours = 0;

                $sunduk_channel = new SundukChannel(
                        $c->id, $c->name, $c->icon_url, $have_archive, $is_protected, $buffering_ms, $timeshift_hours);
                $this->channels->put($sunduk_channel);

                $sunduk_channel->add_group($sunduk_group);
                $sunduk_group->add_channel($sunduk_channel);

                $num_channels++;
                if ($is_protected) {
                    $num_protected++;
                }
                if ($have_archive) {
                    $num_have_archive++;
                }
            }
        }

        hd_print("Sunduk: $num_groups groups and " .
                "$num_channels channels ($num_have_archive have archive, " .
                "$num_protected protected)");
    }

    public function get_tv_info(MediaURL $media_url, &$plugin_cookies) {
        $this->session->ensure_logged_in($plugin_cookies);

        $tv_info = parent::get_tv_info($media_url, &$plugin_cookies);
        $last_channel = $this->session->api_channel_last();
        $this->load_channels($plugin_cookies);
        hd_print('SundukTv->get_tv_info $last_channel '.print_r($last_channel, true));
        $tv_info[PluginTvInfo::initial_channel_id] = $last_channel->channel->id;
        return $tv_info;
        
    }

    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////

    public function get_tv_stream_url($playback_url, &$plugin_cookies) {
        return $this->session->api_get_stream_url($playback_url);
    }

    public function get_tv_playback_url($channel_id, $archive_ts, $protect_code, &$plugin_cookies) {
        $this->ensure_channels_loaded($plugin_cookies);

        $url = sprintf(SUNDUK_GET_URL_URL, Sunduk::$SERVER, $this->session->get_sid_name(),
                $this->session->get_sid(), $channel_id,
                empty($plugin_cookies->protocol)?'hls':$plugin_cookies->protocol);

        if (intval($archive_ts) > 0)
            $url .= "&gmt=$archive_ts";
        if (isset($protect_code) && $protect_code !== '')
            $url .= "&protect_code=$protect_code";

        return $url;
    }

    ///////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////////////

    public function get_day_epg_iterator($channel_id, $day_start_ts, &$plugin_cookies) {
        $this->ensure_channels_loaded($plugin_cookies);

        $day_str = gmstrftime('%d%m%y', $day_start_ts);
        $url = sprintf(SUNDUK_EPG_URL, Sunduk::$SERVER, $this->session->get_sid_name(), $this->session->get_sid(), $channel_id, $day_str);

        try {
            $sunduk_epg = $this->session->api_call($url);
        }
        catch (Exception $e) {
            throw $this->session->dune_api_exception($e, 'EPG request failed.', true);
        }

        $epg_items = array();

        foreach ($sunduk_epg->epg as $epg) {

            $description = strlen($epg->pdescr) == 0 ? '' : $epg->pdescr;

            $start = intval($epg->ut_start);
            $end = -1;
            $epg_items[] = new DefaultEpgItem(
                    $epg->progname, $description, $start, $end);
        }

        return new EpgIterator($epg_items, $day_start_ts - 86400, $day_start_ts + 86400 * 2);
    }

    public function get_archive(MediaURL $media_url) {
        return $this->session->get_archive();
    }

    public function folder_entered(MediaURL $media_url, &$plugin_cookies) {
        if (!isset($media_url->screen_id) ||
                $media_url->screen_id === TvGroupListScreen::ID) {
            $this->session->logout();
        }

        $this->session->ensure_logged_in($plugin_cookies);
    }

    // Hook for adding special group items.
    public function add_special_groups(&$items) { ; }

}
